<?php

namespace Drupal\webform_field_mapper\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Post submission data to a remote URL with field mapping.
 *
 * @WebformHandler(
 *   id = "field_mapper_post",
 *   label = @Translation("Field Mapper Post"),
 *   category = @Translation("Remote Post"),
 *   description = @Translation("Posts submission data to a remote URL with custom field mapping."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class FieldMapperPost extends WebformHandlerBase {

  /**
   * The HTTP client to fetch the remote URL.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClient = $container->get('http_client');
    $instance->tokenManager = $container->get('webform.token_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'remote_url' => '',
        'field_mappings' => [],
        'custom_hidden_fields' => [],
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['remote_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Remote URL'),
      '#description' => $this->t('The remote URL to post data to.'),
      '#default_value' => $this->configuration['remote_url'],
      '#required' => TRUE,
    ];

    $form['field_mappings'] = [
      '#type' => 'details',
      '#title' => $this->t('Field Mappings'),
      '#description' => $this->t('Provide mappings for local fields to remote fields.'),
      '#open' => TRUE,
    ];

    // Get all fields from the webform.
    $webform = $this->getWebform();
    $elements = $webform->getElementsInitializedFlattenedAndHasValue();

    foreach ($elements as $key => $element) {
      $form['field_mappings'][$key] = [
        '#type' => 'textfield',
        '#title' => $element['#title'] ?? $key,
        '#default_value' => $this->configuration['field_mappings'][$key] ?? '',
        '#description' => $this->t('Map @key to a remote field name.', ['@key' => $key]),
      ];
    }

    // Add custom hidden fields configuration.
    $form['custom_hidden_fields'] = [
      '#type' => 'details',
      '#title' => $this->t('Custom Hidden Fields'),
      '#description' => $this->t('Add custom hidden fields that will be sent with the remote post.'),
      '#open' => TRUE,
    ];

    $custom_hidden_fields = $this->configuration['custom_hidden_fields'] ?? [];
    $custom_hidden_fields_count = count($custom_hidden_fields);
    $form_state->set('custom_hidden_fields_count', $custom_hidden_fields_count);

    $form['custom_hidden_fields']['actions'] = [
      '#type' => 'actions',
    ];
    $form['custom_hidden_fields']['actions']['add_field'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another field'),
      '#submit' => ['::addCustomHiddenField'],
      '#ajax' => [
        'callback' => '::addFieldCallback',
        'wrapper' => 'custom-hidden-fields-wrapper',
      ],
    ];

    $form['custom_hidden_fields_container'] = [
      '#type' => 'container',
      '#prefix' => '<div id="custom-hidden-fields-wrapper">',
      '#suffix' => '</div>',
    ];

    for ($i = 0; $i < $custom_hidden_fields_count; $i++) {
      $form['custom_hidden_fields_container']["field_name_$i"] = [
        '#type' => 'textfield',
        '#title' => $this->t('Field Name'),
        '#default_value' => $custom_hidden_fields[$i]['name'] ?? '',
        '#description' => $this->t('The name of the custom hidden field.'),
      ];
      $form['custom_hidden_fields_container']["field_value_$i"] = [
        '#type' => 'textfield',
        '#title' => $this->t('Field Value'),
        '#default_value' => $custom_hidden_fields[$i]['value'] ?? '',
        '#description' => $this->t('The value of the custom hidden field.'),
      ];
    }

    return $form;
  }

  /**
   * Submit handler for adding another custom hidden field.
   */
  public function addCustomHiddenField(array &$form, FormStateInterface $form_state) {
    $custom_hidden_fields_count = $form_state->get('custom_hidden_fields_count');
    $form_state->set('custom_hidden_fields_count', $custom_hidden_fields_count + 1);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Ajax callback for adding another custom hidden field.
   */
  public function addFieldCallback(array &$form, FormStateInterface $form_state) {
    return $form['custom_hidden_fields_container'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['remote_url'] = $form_state->getValue('remote_url');

    // Collect field mappings from form state.
    $field_mappings = [];
    foreach ($form_state->getValue('field_mappings') as $local_field => $remote_field) {
      if (!empty($remote_field)) {
        $field_mappings[$local_field] = $remote_field;
      }
    }
    $this->configuration['field_mappings'] = $field_mappings;

    // Collect custom hidden fields from form state.
    $custom_hidden_fields = [];
    $custom_hidden_fields_count = $form_state->get('custom_hidden_fields_count');
    for ($i = 0; $i < $custom_hidden_fields_count; $i++) {
      $field_name = $form_state->getValue("field_name_$i");
      $field_value = $form_state->getValue("field_value_$i");
      if (!empty($field_name)) {
        $custom_hidden_fields[] = [
          'name' => $field_name,
          'value' => $field_value,
        ];
      }
    }
    $this->configuration['custom_hidden_fields'] = $custom_hidden_fields;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    // Initialize an empty array for the data to be posted.
    $data = [];

    // Get the submitted values from the webform submission.
    $values = $webform_submission->getData();

    // Iterate through the configured field mappings.
    foreach ($this->configuration['field_mappings'] as $local_field => $remote_field) {
      // Check if the local field exists in the submitted values.
      if (isset($values[$local_field])) {
        // Map the local field value to the corresponding remote field.
        $data[$remote_field] = $values[$local_field];
      }
    }

    // Add custom hidden fields to the data.
    foreach ($this->configuration['custom_hidden_fields'] as $custom_field) {
      // Add each custom hidden field to the data array.
      $data[$custom_field['name']] = $custom_field['value'];
    }

    try {
      // Make a POST request to the remote URL with the data.
      $this->httpClient->post($this->configuration['remote_url'], [
        'json' => $data,
      ]);
    }
    catch (\Exception $e) {
      // Log any exceptions that occur during the HTTP request.
      \Drupal::logger('webform_field_mapper')->error($e->getMessage());
    }
  }
}
